@@ -35,7 +35,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.63 $ $Date: 2007/11/16 05:29:27 $";
+static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.65 $ $Date: 2008/06/14 04:38:32 $";
 #endif /* DEBUG */
 
 #ifndef NSSPKI_H
@@ -74,10 +74,11 @@ nssCertificate_Create (
 {
     PRStatus status;
     NSSCertificate *rvCert;
-    /* mark? */
+    nssArenaMark * mark;
     NSSArena *arena = object->arena;
     PR_ASSERT(object->instances != NULL && object->numInstances > 0);
     PR_ASSERT(object->lockType == nssPKIMonitor);
+    mark = nssArena_Mark(arena);
     rvCert = nss_ZNEW(arena, NSSCertificate);
     if (!rvCert) {
 	return (NSSCertificate *)NULL;
@@ -93,13 +94,19 @@ nssCertificate_Create (
                                                   &rvCert->issuer,
                                                   &rvCert->serial,
                                                   &rvCert->subject);
-    if (status != PR_SUCCESS) {
-	return (NSSCertificate *)NULL;
-    }
-    /* all certs need an encoding value */
-    if (rvCert->encoding.data == NULL) {
+    if (status != PR_SUCCESS ||
+	!rvCert->encoding.data ||
+	!rvCert->encoding.size ||
+	!rvCert->issuer.data ||
+	!rvCert->issuer.size ||
+	!rvCert->serial.data ||
+	!rvCert->serial.size) {
+	if (mark)
+	    nssArena_Release(arena, mark);
 	return (NSSCertificate *)NULL;
     }
+    if (mark)
+	nssArena_Unmark(arena, mark);
     return rvCert;
 }
 
@@ -121,7 +128,6 @@ nssCertificate_Destroy (
 {
     nssCertificateStoreTrace lockTrace = {NULL, NULL, PR_FALSE, PR_FALSE};
     nssCertificateStoreTrace unlockTrace = {NULL, NULL, PR_FALSE, PR_FALSE};
-    PRBool locked = PR_FALSE;
 
     if (c) {
 	PRUint32 i;
@@ -134,7 +140,6 @@ nssCertificate_Destroy (
 	/* --- LOCK storage --- */
 	if (cc) {
 	    nssCertificateStore_Lock(cc->certStore, &lockTrace);
-            locked = PR_TRUE;
 	} else {
 	    nssTrustDomain_LockCertCache(td);
 	}
@@ -144,8 +149,6 @@ nssCertificate_Destroy (
 		nssCertificateStore_RemoveCertLOCKED(cc->certStore, c);
 		nssCertificateStore_Unlock(cc->certStore, &lockTrace,
                                            &unlockTrace);
-                nssCertificateStore_Check(&lockTrace, &unlockTrace);
-
 	    } else {
 		nssTrustDomain_RemoveCertFromCacheLOCKED(td, c);
 		nssTrustDomain_UnlockCertCache(td);
@@ -163,15 +166,11 @@ nssCertificate_Destroy (
 		nssCertificateStore_Unlock(cc->certStore,
 					   &lockTrace,
 					   &unlockTrace);
-		nssCertificateStore_Check(&lockTrace, &unlockTrace);
 	    } else {
 		nssTrustDomain_UnlockCertCache(td);
 	    }
 	}
     }
-    if (locked) {
-        nssCertificateStore_Check(&lockTrace, &unlockTrace);
-    }
     return PR_SUCCESS;
 }
 
